;;; -*- Mode: LISP; Syntax: Common-lisp; Package: DTP; Base: 10 -*-

;;;----------------------------------------------------------------------------
;;;
;;;	File		Caching.Lisp
;;;	System		Don's Theorem Prover
;;;
;;;	Written by	Don Geddis (Geddis@CS.Stanford.Edu)

(in-package "DTP")

;;;----------------------------------------------------------------------------
;;;
;;;	Public

;;;----------------------------------------------------------------------------

(defun find-subgoal (conjunct)
  #+dtp-types (declare (type dtp-conjunct conjunct))
  "Return (1) an old or new subgoal SAMEP as LITERAL, (2) binding list or nil"
  (let ((bl nil)
	literal sg )
    (setq literal
      (literal-plug
       (slot-value conjunct 'literal)
       (slot-value conjunct 'binding-list) ))
    (setq literal (nsimplify-terms literal)) ; Hook for term rewriting
    (setq sg (find-stored-subgoal literal))
    (if sg
	(progn
	  (setq bl
	    (dtp-samep
	     (literal-terms (slot-value sg 'literal))
	     (literal-terms literal)
	     t ))
	  #+dtp-trace
	  (when (find :proofs *trace*)
	    (indent-line)
	    (format *debug-io* "Attaching to existing subgoal ")
	    (print-literal-node (slot-value sg 'literal) :s *debug-io*)
	    (format *debug-io* "~%") )
	  (let ((parent (subgoal-parent-of conjunct))
		alternate-height )
	    (if parent
		(setq alternate-height (1+ (slot-value parent 'depth)))
	      (setq alternate-height 0) )
	    (when (< alternate-height (slot-value sg 'depth))
	      (setf (slot-value sg 'depth) alternate-height)
	      #+dtp-trace
	      (when (find :proofs *trace*)
		(indent-line)
		(format *debug-io* "And lowering subgoal depth to ~D~%"
			alternate-height ))
	      )))
      (progn
	(setq sg (make-instance 'dtp-subgoal :literal literal))
	(with-slots (parent-conjunction) conjunct
	  (with-slots (parent-subgoal) parent-conjunction
	    (setf (slot-value sg 'parent-subgoal) parent-subgoal)
	    (if parent-subgoal
		(setf (slot-value sg 'depth)
		  (1+ (slot-value parent-subgoal 'depth)) )
	      (setf (slot-value sg 'depth) 0) )))
	(memo-subgoal sg)
	#+dtp-trace
	(when (find :proofs *trace*)
	  (indent-line)
	  (format *debug-io* "Creating new subgoal ")
	  (print-literal-node literal :s *debug-io*)
	  (format *debug-io* "~%") )))
    (values sg bl) ))

;;;----------------------------------------------------------------------------
;;;
;;;	Private

;;;----------------------------------------------------------------------------

(defun find-stored-subgoal (literal)
  #+dtp-types (declare (type literal-node literal))
  (unless *use-caching*
    (return-from find-stored-subgoal nil) )
  (let (sg-list)
    (setq sg-list
      (gethash (literal-relation literal) (proof-subgoal-index *proof*)) )
    (find literal sg-list
	  :test #'(lambda (lit sg)
		    (literal-samep lit (slot-value sg 'literal)) ))
    ))

;;;----------------------------------------------------------------------------

(defun memo-subgoal (subgoal)
  #+dtp-types (declare (type dtp-subgoal subgoal))
  (unless *use-caching*
    (return-from memo-subgoal) )
  (let ((relation (literal-relation (slot-value subgoal 'literal)))
	(table (proof-subgoal-index *proof*)) )
    (if (gethash relation table)
	(push subgoal (gethash relation table))
      (setf (gethash relation table) (list subgoal)) )
    ))

;;;----------------------------------------------------------------------------
