;;; -*- Mode: LISP; Syntax: Common-lisp; Package: DTP; Base: 10 -*-

;;;----------------------------------------------------------------------------
;;;
;;;	File		Conjunct.Lisp
;;;	System		Don's Theorem Prover
;;;
;;;	Written by	Don Geddis (Geddis@CS.Stanford.Edu)

(in-package "DTP")

;;;----------------------------------------------------------------------------
;;;
;;;	Public

;;;----------------------------------------------------------------------------

(defmethod expand ((conjunct dtp-conjunct))
  (with-slots (literal parent-conjunction binding-list transform-binding-list
	       answer-count subgoal nogoods ) conjunct
    (when (eq subgoal :uninitialized)
      (multiple-value-bind (new-subgoal tbl) (find-subgoal conjunct)
        (setq subgoal new-subgoal)
        (setq transform-binding-list tbl)
	(setq nogoods nil)
	(agenda-add subgoal) ))
    (with-slots (answers conjuncts-to-propogate-to) subgoal
      (let ((answer (nth answer-count answers)))
        (when (and transform-binding-list answer)
          (setq answer (copy-answer answer))
	  (setf (answer-binding-list answer)
	        (plug (answer-binding-list answer) transform-binding-list) ))
        (cond
         (answer
	  (incf answer-count)
	  (propogate answer parent-conjunction) )
         ((exhausted-p subgoal)
	  (propogate :not-an-answer parent-conjunction) )
	 ((and (virgin-subgoal-p subgoal)
	       (not (find conjunct conjuncts-to-propogate-to)) )
	  (add-to-end conjunct conjuncts-to-propogate-to) )
         (t                             ; Create a new forward-inf conjunction
	  (let ((new-conj (make-instance-conjunction parent-conjunction))
		(parent-subgoal
		 (slot-value parent-conjunction 'parent-subgoal) )
                head-conjunct )

            ;; Set up the new conjunction
	    (when new-conj
	      (if parent-subgoal
		  (push new-conj
			(slot-value
			 parent-subgoal 'forward-inference-conjunctions ))
		(push new-conj
		      (proof-forward-inference-conjunctions *proof*) ))
	      (setq head-conjunct (first (slot-value new-conj 'list)))
	      (add-to-end head-conjunct conjuncts-to-propogate-to) )
	    #+dtp-trace
	    (add-to-end
	     new-conj (slot-value parent-conjunction 'instance-conjunctions) )
            
            ;; Continue with the original conjunction
	    (reset conjunct)
	    (propogate :not-an-answer parent-conjunction) )
	  )))
      )))

#+dtp-trace
(defmethod expand :around ((conjunct dtp-conjunct))
  "For proof tracing"
  (when (find :conjunct-proofs *trace*)
    (indent-line)
    (format *debug-io* "Expanding conjunct ")
    (print-literal-node
     (literal-plug (slot-value conjunct 'literal)
		   (slot-value conjunct 'binding-list) )
     :s *debug-io*)
    (format *debug-io* "~%") )
  (call-next-method) )

;;;----------------------------------------------------------------------------

(defmethod propogate (answer (conjunct dtp-conjunct))
  #+dtp-types (declare (type answer answer))
  (with-slots
      (parent-conjunction transform-binding-list answer-count ranswer-count)
      conjunct
    (when (and (>= (slot-value parent-conjunction 'stack-pointer) 0)
	       (eq (nth (slot-value parent-conjunction 'stack-pointer)
			(slot-value parent-conjunction 'list) )
		   conjunct ))
      (if (eq answer :not-an-answer)
	  (propogate :not-an-answer parent-conjunction)
	(let ((new-answer answer))
	  (when transform-binding-list
	    (setq new-answer (copy-answer answer))
	    (setf (answer-binding-list new-answer)
	      (plug (answer-binding-list answer) transform-binding-list) ))
	  (incf answer-count)
	  (when (typep answer 'reduction-answer)
	    (incf ranswer-count) )
	  (propogate new-answer parent-conjunction) ))
      )))

#+dtp-trace
(defmethod propogate :around (answer (conjunct dtp-conjunct))
  "For proof tracing"
  #+dtp-types (declare (type answer answer))
  (when (find :conjunct-proofs *trace*)
    (indent-line)
    (format *debug-io* "Propogating ~S to conjunct " answer)
    (print-literal-node
     (literal-plug (slot-value conjunct 'literal)
		   (slot-value conjunct 'binding-list) )
     :s *debug-io* )
    (format *debug-io* "~%") )
  (call-next-method) )

;;;----------------------------------------------------------------------------
;;;
;;;	Private

;;;----------------------------------------------------------------------------

(defmethod reset ((conjunct dtp-conjunct))
  (with-slots (subgoal answer-count nogoods #+dtp-trace used-subgoals) conjunct
    (unless (eq subgoal :uninitialized)
      #+dtp-trace (add-to-end subgoal used-subgoals)
      (setq subgoal :uninitialized) )
    (setq answer-count 0)
    (setq nogoods :uninitialized) ))

;;;----------------------------------------------------------------------------
