;;; -*- Mode: LISP; Syntax: Common-lisp; Package: DTP; Base: 10 -*-

;;;----------------------------------------------------------------------------
;;;
;;;	File		Variables.Lisp
;;;	System		Don's Theorem Prover
;;;
;;;	Written by	Don Geddis (Geddis@CS.Stanford.Edu)

(in-package "DTP")

;;;----------------------------------------------------------------------------
;;;
;;;	Parameters

(defparameter *dtp-package* (find-package "DTP"))

(defparameter *test-ignore-answer* '**)
(defparameter *logic-samples*
    '(tests examples overview complete cornell caching
      frapps steamroller others ))

;;;----------------------------------------------------------------------------
;;;
;;;	Initialization (reset restores these values)

(defparameter *dtp-initial-values*
    '(
      (*theory* . global)
      
      ;; Inference Options
      (*use-negated-goal* . t)
      (*use-subgoal-inference* . t)
      (*use-contrapositives* . t)
      (*use-reduction* . t)
      (*use-pure-literal-elimination* . t)
      (*use-backjumping* . t)
      (*use-caching* . t)
      (*use-iap* . nil)
      (*use-residue* . t)
      (*use-procedural-attachments* . t)
      (*use-reordering* . t)

      ;; Iteration
      (*use-subgoal-cutoffs* . nil)
      (*initial-subgoal-depth* . 1)
      (*subgoal-depth-skip* . 1)
      (*subgoal-maximum-depth* . nil)
      (*use-function-cutoffs* . nil)
      (*initial-function-depth* . 1)
      (*function-depth-skip* . 1)
      (*function-maximum-depth* . nil)

      ;; Residue
      (*assumables* . nil)
      (*consistency-check* . nil)
      
      ;; Tracing
      #+dtp-trace (*trace* . (:file-load :tests))
      #+dtp-trace (*trace-keywords* .
		   (:file-load :tests :answers :subgoals :proofs
		    :conjunct-proofs :backtrack :cutoffs ))
      #+dtp-trace (*single-step* . nil)

      ;; Internal
      (*node-id-count* . 0)
      (*all-gensymed-variables* . nil)
      ))

(defun initial-value (variable)
  (rest (assoc variable *dtp-initial-values*)) )

;;;----------------------------------------------------------------------------
;;;
;;;	Variables

;; Display form
(defvar *display-logic-as-lists* nil)
(defvar *show-renamed-variables* nil)
(defvar *graphic-display* t "Toggle: Output in Postscript [T] or text?")

;; Misc
(defvar *theory*)
(defvar *last-proof*)
(defvar *test-proofs*)

;; Inference engine
(defvar *use-negated-goal*)
(defvar *use-subgoal-inference*)
(defvar *use-contrapositives*)
(defvar *use-reduction*)
(defvar *use-pure-literal-elimination*)
(defvar *use-backjumping*)
(defvar *use-caching*)
(defvar *use-iap*)
(defvar *use-residue*)
(defvar *use-procedural-attachments*)
(defvar *use-reordering*)

;; Iteration
(defvar *use-subgoal-cutoffs*)
(defvar *initial-subgoal-depth*)
(defvar *subgoal-depth-skip*)
(defvar *subgoal-maximum-depth*)
(defvar *use-function-cutoffs*)
(defvar *initial-function-depth*)
(defvar *function-depth-skip*)
(defvar *function-maximum-depth*)

;; Residue
(defvar *assumables*)
(defvar *consistency-check*)

;; Tracing
#+dtp-trace (defvar *trace*)
#+dtp-trace (defvar *proof-line-count*)
#+dtp-trace (defvar *depth*)
#+dtp-trace (defvar *subgoal-map*)
#+dtp-trace (defvar *single-step*)

;; Graphic Display
#+dtp-trace (defparameter *dot-command*
		#+customize-dfg "dot.standalone"
		#-customize-dfg "dot" )
#+dtp-trace (defparameter *dot-file* "/tmp/proof.dot")
#+dtp-trace (defparameter *ps-viewer-command* "ghostview")
#+dtp-trace (defparameter *ps-file* "/tmp/proof.ps")
#+dtp-trace (defvar *display-one-page* t)
#+dtp-trace (defvar *display-landscape* t)
#+dtp-trace (defvar *display-title* t)
#+dtp-trace (defvar *display-query* t)
#+dtp-trace (defvar *display-color* nil)

;; Internal
(defvar *node-id-count*)
(defvar *all-gensymed-variables* nil)	; So we can unintern them on reset
(defvar *proof*)			; Dynamic variable, set to (make-proof)
(defvar *goal-node-id-count*)

;;;----------------------------------------------------------------------------

(defparameter *user-setable-variables*
    `(("General"
       *theory* *assumables* *consistency-check* )
      ("Inference Engine"
       *use-negated-goal* *use-subgoal-inference* *use-contrapositives*
       *use-reduction* *use-pure-literal-elimination* *use-backjumping*
       *use-caching* *use-iap* *use-residue* *use-procedural-attachments*
       *use-reordering* )
      ("Iteration"
       *use-subgoal-cutoffs* *initial-subgoal-depth* *subgoal-depth-skip*
       *subgoal-maximum-depth* *use-function-cutoffs* *initial-function-depth*
       *function-depth-skip* *function-maximum-depth* )
      ("Display form"
       *display-logic-as-lists* *show-renamed-variables* *graphic-display* )
      #+dtp-trace
      ("Graphic Display"
       *display-one-page* *display-landscape* *display-title* *display-query*
       *display-color* )
      #+dtp-trace
      ("Tracing"
       *trace* *single-step* )
      ))

;;;----------------------------------------------------------------------------

(defun reset-variables ()
  (dolist (pair *dtp-initial-values*)
    (set (first pair) (rest pair)) ))

(eval-when (load eval)
  (reset-variables) )

;;;----------------------------------------------------------------------------
